/**
 * The Apple class represents an apple that can be consumed by the kobold,
 * which increases its statuses.
 */
function Apple() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Apple";
    var itemDescription = "A large, reddish apple. Delicious! Eat it.";
    var baseValue = 4;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the consume popup contents.
     * @returns {string} The consume popup contents.
     */
    function buildPopupContents() {
        var content = ` [st]${itemName}[/st]
                        [p]
                            You chomp down on the apple.
                            It's nice and juicy on the inside.
                            Ripe, but not [i]too[/i] ripe.
                            Sweetened to near-perfection.
                        [/p]
                        [p]
                            You savor the fruity flavor while chewing off bite after bite.
                            Before long, the apple is gone.
                            You toss away the core, having heard that it contains some kind of poison.
                            You are left feeling just a bit less hungry than you were before,
                            and a little better in general.
                        [/p]`;
        return global.parser.parse(content);
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.APPLE;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.CONSUMABLE;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        var toReturn = caps ? itemName : itemName.toLowerCase();
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        return itemDescription;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        return baseValue;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        return "width: 160px;";
    };

    /**
     * CONSUMABLE-REQUIRED function.
     * Consumes this consumable.
     */
    this.consume = function () {
        global.kobold.status.physical.increase(5, "Ate an apple");
        global.kobold.status.mental.increase(5, "Ate an apple");
        global.kobold.status.stamina.increase(5, "Ate an apple");
        global.kobold.status.exertion.decrease(5, "Ate an apple");
        var contents = buildPopupContents();
        contents += global.ui.change.buildNotification([
            new Change("Food", "Health", +5), 
            new Change("Food", "Sanity", +5),
            new Change("Food", "Stamina", +5),
            new Change("Food", "Sexual exhaustion", -5, true)
        ]);
        global.ui.showFullscreenPopup(contents);
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {AgilityPotion-stripped} potion An AgilityPotion object, stripped of private members and all functions.
     */
    this.load = function (potion) {

    };
}
