/**
 * The LeatherPants class represents a pair of leather pants that can be worn by the kobold.
 */
function LeatherPants() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;

    var itemName = "Leather Pants";
    var itemDescription = "A pair of pants made out of sturdy leather, both flexible and durable.";
    var baseValue = 40;
    var offensiveValue = 0;
    var defensiveValue = 4;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // ITEM-REQUIRED item-enum for construction purposes.
    this.id = ItemEnum.LEATHERPANTS;
    // ITEM-REQUIRED type-enum for tooltip purposes.
    this.type = ItemTypeEnum.GEAR;
    // GEAR-REQUIRED slot-enum.
    this.slot = SlotEnum.PANTS;

    // What this pair of pants is stained with.
    this.isStainedWith = "";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the name of this item.
     * @param {boolean} caps Whether or not the name should be capitalized.
     * @returns {string} The name of this item.
     */
    this.name = function (caps) {
        if (that.stained()) {
            var built = that.isStainedWith + "-stained " + itemName;
            var toReturn = caps ? built : built.toLowerCase();
            return toReturn;
        } else {
            var toReturn = caps ? itemName : itemName.toLocaleLowerCase();
            return toReturn;
        }
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns the description of this item.
     * @returns {string} The description of this item.
     */
    this.description = function () {
        var toReturn = that.stained() ? itemDescription + " They reek of " + that.isStainedWith : itemDescription;
        return toReturn;
    };

    /**
     * ITEM-REQUIRED function.
     * Calculates and returns the price (= value) of this item.
     * @returns {int} The price of this pair of pants, in coins.
     */
    this.price = function () {
        var calculatedPrice = that.stained() ? baseValue - 3 : baseValue;
        return calculatedPrice;
    };

    /**
     * ITEM-REQUIRED function.
     * Builds and returns a custom tooltip style for this item.
     * @returns {string} The custom style.
     */
    this.toolTipStyle = function () {
        if (!that.stained()) {
            return "width: 175px;";
        } else {
            return "width: 200px;";
        }
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the offensive value of this piece of gear.
     * @returns {int} The offensive value of this piece of gear.
     */
    this.offence = function () {
        return offensiveValue;
    };

    /**
     * GEAR-REQUIRED function.
     * Calculates and returns the defensive value of this piece of gear.
     * @returns {int} The defensive value of this piece of gear.
     */
    this.defence = function () {
        return defensiveValue;
    };

    /**
     * PANTS-required function.
     * Stains the pants.
     * @param {string} what What the pants get stained with. Should always start with a capital letter.
     */
    this.stain = function (what) {
        that.isStainedWith = what;
    };

    /**
     * PANTS-required function.
     * Returns true if the pants are stained, and false otherwise.
     * @returns {boolean} True if the pants are stained, and false otherwise.
     */
    this.stained = function () {
        return that.isStainedWith !== "";
    };

    /**
     * PANTS-required function.
     * Returns what the pants are stained with.
     * @returns {string} What the pants are stained with.
     */
    this.stainedWith = function () {
        return that.isStainedWith;
    };

    /**
     * PANTS-required function.
     * Cleans the pants, removing any stains.
     */
    this.clean = function () {
        that.isStainedWith = "";
    };

    /**
     * PANTS-required function.
     * Returns true if the pants cover the groin, and false otherwise.
     * @returns {boolean} True if the pants cover the groin, and false otherwise.
     */
    this.coversGroin = function () {
        return true;
    };

    /**
     * PANTS-required function.
     * Returns true if the pants cover the rear, and false otherwise.
     * @returns {boolean} True if the pants cover the rear, and false otherwise.
     */
    this.coversRear = function () {
        return true;
    };

    /**
     * PANTS-required function.
     * Returns true if the pants can be torn, and false otherwise.
     * @returns {boolean} True if the pants can be torn, and false otherwise.
     */
    this.canBeTorn = function () {
        return true;
    };

    /**
     * PANTS-required function.
     * Returns true if the pants are a mismatch for the kobold's current gender.
     * @returns {boolean} True if the pants are a mismatch for the kobold's current gender, false otherwise.
     */
    this.wrongGender = function () {
        return false; // Linen pants fit all genders.
    };

    /**
     * ITEM-REQUIRED function.
     * Loads this object from a type-stripped version thereof.
     * @param {LinenPants-stripped} pants A LinenPants object, stripped of private members and all functions.
     */
    this.load = function (pants) {
        that.isStainedWith = pants.isStainedWith;
    };
}
