/**
 * Information about the class.
 */
function ImageBackground() {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var id = "imagebackground";
    var cls = "imagebackground";
    var src = "";
    var alt = "image background";
    var dur = 1000;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}
    /**
     * Hides and deletes any currently present background images, then calls the passed function.
     * Calls the function immediately if there is no currently present background image.
     * @param {function} fun Function to call after hiding. May be called immediately.
     */
    function hideAndThen(fun) {
        if (src) {
            var elem = $("#" + id);
            elem.removeAttr("id");
            elem.stop(true);
            src = "";
            elem.fadeOut(dur, function () {
                elem.remove();
                fun();
            });
        } else {
            fun();
        }
    }

    /**
     * Loads the image with the given url, and then shows it once it's fully loaded.
     * @param {string} url URL of the image relative to the site root.
     */
    function loadAndShowImage(url) {
        var img = new Image();
        img.id = id;
        img.className = cls;
        img.alt = alt;
        img.style = "display: none;";
        img.onload = function () {
            $(this).fadeIn(dur);
        };
        img.src = url;
        src = url;
        $("body").prepend(img);
    }

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Shows the given url as background image, fading it in properly.
     * Hides any currently present background images first.
     * @param {string} url URL of the image relative to the site root.
     */
    this.show = function (url) {
        if (src === url) {
            return;
        }
        hideAndThen(function () {
            loadAndShowImage(url);
        });
    };

    /**
     * Hides and deletes any present background images.
     */
    this.hide = function () {
        hideAndThen(function () {});
    };
}

//