/**
 * Cart section of Enslavement, Night One.
 * @param {CaravanConfinementController} controller The Enslavement, Night One controller.
 */
function CaravanConfinementCart(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = ctrl.postTight;
    var subController = ctrl.cart;

    var firstId = arguments.callee.name + "ButtonOne";
    var secondId = arguments.callee.name + "ButtonTwo";
    var thirdId = arguments.callee.name + "ButtonThree";
    var fourthId = arguments.callee.name + "ButtonFour";
    var fifthId = arguments.callee.name + "ButtonFive";
    var sixthId = arguments.callee.name + "ButtonSix";

    var firstNext = new CaravanConfinementBreak(ctrl);
    var secondNext = new CaravanConfinementSlip(ctrl);
    var thirdNext = new CaravanConfinementBang(ctrl);
    var fourthNext = new CaravanConfinementCount(ctrl);
    var fifthNext = new CaravanConfinementGaze(ctrl);
    var sixthNext = new CaravanConfinementTalk(ctrl);

    var firstChoice = CaravanConfinementCartChoice.BREAK;
    var secondChoice = CaravanConfinementCartChoice.SLIP;
    var thirdChoice = CaravanConfinementCartChoice.BANG;
    var fourthChoice = CaravanConfinementCartChoice.COUNT;
    var fifthChoice = CaravanConfinementCartChoice.GAZE;
    var sixthChoice = CaravanConfinementCartChoice.TALK;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var male = ctrl.misc.baseGender() === GenderEnum.MALE;
        var fucked = global.kobold.history.shared.enslavement.captureFucked;
        var hole = "rear end";
        if (male && fucked) {
            hole = "cockslit";
        }
        if (!male && !fucked) {
            hole = "cunt";
        }
        var toReturn = `[p]
                            Satisfied with his [i]discovery[/i], the caravan master removes his finger from your ${hole}, and rises to his feet.
                            "[color=brown]That's all I needed to know.
                            Shove'em in one of the empty carts near the back.
                            Station an extra watchman to ensure our little [i]guest[/i] stays put during the night.
                            We'll split off from the main caravan once we get close enough to the city
                            for the guards to ensure your men won't go running off with the gold.[/color]"
                        [/p]
                        [p]
                            Fenrir, or so the human called him earlier, scoffs at the notion that his group would break a bargain without being told to do so.
                            After a bit of uninteresting back and forth idle chatter,
                            the wolf grabs you by the collar, much like before,
                            and sets off for the rear end of the convoy,
                            tugging you along with.
                        [/p]
                        [p]
                            Arriving at the second-to-last transport carriage, the canine comes to a halt.
                            "[color=gblue]This is your stop.[/color]" he speaks, unlocking and opening the cart.
                            Grabbing a piece of rope hanging off the side of the vehicle, the buff lupine deftly ties your wrists together behind your back.
                            You don't even bother [i]trying[/i] to resist.
                            Your hands safely secured, the wolf casually lifts you up off the ground.
                            In a surprisingly gentle fashion, he rests your body down on the wooden carriage floor.
                        [/p]
                        [p]
                            "[color=gblue]Try to get some sleep.
                            Wouldn't want you to look weary during the auction tomorrow, now would we?
                            Remember: if you don't sell, your ass is [b]mine[/b].
                            And I share [b]everything[/b] with the rest of my crew.[/color]"
                            His mixture between a [i]threat[/i] and a sarcastic [i]pseudo-invitation[/i] shared,
                            the wolf closes the cart's sole door, granting you some peace and quiet after a long and arduous day.
                        [/p]
                        [p]
                            You're tired and exhausted, both mentally and physically, but you think you have enough energy left for one final act before going to sleep.
                            Outside of the cart, you hear footsteps.
                            Probably your personal guard, here to ensure you won't be [i]leaving[/i].
                            As if you could escape with your hands tied.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildFirstChoice() {
        var disabled = subController.has() ? "disabled" : "";
        var isSelected = subController.get() === firstChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${firstId}" ${disabled} style="width: 190px;">Break bindings</button>`;

        var tooltip = global.parser.parse(`Attempt to break the rope holding your hands together.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice" style="width: 190px;">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildSecondChoice() {
        var disabled = subController.has() ? "disabled" : "";
        var isSelected = subController.get() === secondChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${secondId}" ${disabled} style="width: 190px;">Slip from bindings</button>`;

        var tooltip = global.parser.parse(`Try to remove your hands from the rope binding them together.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice" style="width: 190px;">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildThirdChoice() {
        var disabled = subController.has() ? "disabled" : "";
        var isSelected = subController.get() === thirdChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${thirdId}" ${disabled} style="width: 190px;">Use your head</button>`;

        var tooltip = global.parser.parse(`Wantonly bang your forehead against the side of the cart. Maybe the wood'll break.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice" style="width: 190px;">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }
    /**
     * Builds and returns the choice button.
     */
    function buildFourthChoice() {
        var disabled = subController.has() ? "disabled" : "";
        var isSelected = subController.get() === fourthChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${fourthId}" ${disabled} style="width: 190px;">Count bars</button>`;

        var tooltip = global.parser.parse(`Count your makeshift cell's iron bars to pass the time.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 200px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice" style="width: 190px;">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildFifthChoice() {
        var disabled = subController.has() ? "disabled" : "";
        var isSelected = subController.get() === fifthChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${fifthId}" ${disabled} style="width: 190px;">Stargaze</button>`;

        var tooltip = global.parser.parse(`You can see some of the stars through the cart's small, barred windows. Let the night sky calm you.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 190px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice" style="width: 190px;">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildSixthChoice() {
        var disabled = subController.has() ? "disabled" : "";
        var isSelected = subController.get() === sixthChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${sixthId}" ${disabled} style="width: 190px;">Talk</button>`;

        var tooltip = global.parser.parse(`Strike a conversation with the sentry posted outside of your prison.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice" style="width: 190px;">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You:</div>`;
        choice += `<div class="choicerow" style="margin-bottom: 10px;">`;
        choice += buildFirstChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildSecondChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildThirdChoice();
        choice += `</div>`;
        choice += `<div class="choicerow">`;
        choice += buildFourthChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildFifthChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildSixthChoice();
        choice += `</div>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + firstId), $("#" + secondId), $("#" + thirdId), $("#" + fourthId), $("#" + fifthId), $("#" + sixthId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerFirstListener() {
        $("#" + firstId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(firstChoice);
            loadNextPart(firstNext);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerSecondListener() {
        $("#" + secondId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(secondChoice);
            loadNextPart(secondNext);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerThirdListener() {
        $("#" + thirdId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(thirdChoice);
            loadNextPart(thirdNext);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerFourthListener() {
        $("#" + fourthId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(fourthChoice);
            loadNextPart(fourthNext);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerFifthListener() {
        $("#" + fifthId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(fifthChoice);
            loadNextPart(fifthNext);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerSixthListener() {
        $("#" + sixthId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(sixthChoice);
            loadNextPart(sixthNext);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += buildChoice();
        part += `</div>`;
        part += firstNext.build();
        part += secondNext.build();
        part += thirdNext.build();
        part += fourthNext.build();
        part += fifthNext.build();
        part += sixthNext.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        firstNext.registerListeners();
        secondNext.registerListeners();
        thirdNext.registerListeners();
        fourthNext.registerListeners();
        fifthNext.registerListeners();
        sixthNext.registerListeners();

        if (subController.has()) {
            return;
        }

        registerFirstListener();
        registerSecondListener();
        registerThirdListener();
        registerFourthListener();
        registerFifthListener();
        registerSixthListener();
    };
}

var CaravanConfinementCartChoice = {
    BREAK: 1,
    SLIP: 2,
    BANG: 3,
    COUNT: 4,
    GAZE: 5,
    TALK: 6
};
