/**
 * Result section of Enslavement, Noon Two.
 * @param {SlaveNoonTwoController} controller The Enslavement, Noon Two controller.
 */
function SlaveNoonTwoResults(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = ctrl.preResult;
    var subController = ctrl.result;
    var changeController = ctrl.result;

    var firstId = arguments.callee.name + "ButtonOne";
    var secondId = arguments.callee.name + "ButtonTwo";
    var thirdId = arguments.callee.name + "ButtonThree";
    var fourthId = arguments.callee.name + "ButtonFour";

    var firstNext = new SlaveNoonTwoNobody(ctrl);
    var secondNext = new SlaveNoonTwoSeki(ctrl);
    var thirdNext = new SlaveNoonTwoFarmer(ctrl);
    var fourthNext = new SlaveNoonTwoVixen(ctrl);

    var firstChoice = SlaveNoonTwoResultsChoice.NOBODY;
    var secondChoice = SlaveNoonTwoResultsChoice.SEKI;
    var thirdChoice = SlaveNoonTwoResultsChoice.FARMER;
    var fourthChoice = SlaveNoonTwoResultsChoice.VIXEN;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        if (!ctrl.resultComplianceStorage.has()) {
            ctrl.resultComplianceStorage.set(global.kobold.history.shared.enslavement.nightOne.complianceRating
                    + global.kobold.history.shared.enslavement.noonTwo.complianceRating
                    + ctrl.resultBodyMisc.breasts()
                    - ctrl.resultBodyMisc.dick());
        }

        var toReturn = `[p]
                            People start giving proper offers.
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildFirstChoice() {
        var disabled = subController.has() ? "disabled" : "";
        var isSelected = subController.get() === firstChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${firstId}" ${disabled} style="width: 190px;">Nobody</button>`;

        var tooltip = global.parser.parse(`The caravan master's not going to like this.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice" style="width: 190px;">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildSecondChoice() {
        var fail = ctrl.gag.get();
        var disabled = subController.has() || fail ? "disabled" : "";
        var isSelected = subController.get() === secondChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${secondId}" ${disabled} style="width: 190px;">Seki</button>`;

        var tooltip = global.parser.parse(`The elderly bird you've known since you were little comes to your rescue!`);
        if (fail) {
            tooltip = global.parser.parse(`You are gagged, and can not call for help.`);
        }
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice" style="width: 190px;">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildThirdChoice() {
        var succeed = ctrl.resultComplianceStorage.get() > 15;
        var disabled = subController.has() || !succeed ? "disabled" : "";
        var isSelected = subController.get() === thirdChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${thirdId}" ${disabled} style="width: 190px;">Farmer</button>`;

        var tooltip = global.parser.parse(`The farmer's son looks fit to nag his father until he relents to buying you.`);
        if (!succeed) {
            tooltip = global.parser.parse(`The farmer is looking for a good girl. You don't interest him a lot.`);
        }
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice" style="width: 190px;">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }
    /**
     * Builds and returns the choice button.
     */
    function buildFourthChoice() {
        var succeed = ctrl.resultComplianceStorage.get() < 8;
        var disabled = subController.has() || !succeed ? "disabled" : "";
        var isSelected = subController.get() === fourthChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${fourthId}" ${disabled} style="width: 190px;">Vixen</button>`;

        var tooltip = global.parser.parse(`That vixen has a wicked grin on her face. You don't like it one bit.`);
        if (!succeed) {
            tooltip = global.parser.parse(`The vixen is looking for a bad boy. You don't interest her a lot.`);
        }
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 160px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice" style="width: 190px;">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You:</div>`;
        choice += `<div class="choicerow" style="margin-bottom: 10px;">`;
        choice += buildFirstChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildSecondChoice();
        choice += `</div>`;
        choice += `<div class="choicerow">`;
        choice += buildThirdChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildFourthChoice();
        choice += `</div>`;
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + firstId), $("#" + secondId), $("#" + thirdId), $("#" + fourthId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerFirstListener() {
        $("#" + firstId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(firstChoice);
            loadNextPart(firstNext);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerSecondListener() {
        $("#" + secondId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(secondChoice);
            loadNextPart(secondNext);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerThirdListener() {
        $("#" + thirdId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(thirdChoice);
            loadNextPart(thirdNext);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerFourthListener() {
        $("#" + fourthId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(fourthChoice);
            loadNextPart(fourthNext);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += buildChoice();
        part += `</div>`;
        part += firstNext.build();
        part += secondNext.build();
        part += thirdNext.build();
        part += fourthNext.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        firstNext.registerListeners();
        secondNext.registerListeners();
        thirdNext.registerListeners();
        fourthNext.registerListeners();

        if (subController.has()) {
            return;
        }

        registerFirstListener();
        registerSecondListener();
        registerThirdListener();
        registerFourthListener();
    };
}

var SlaveNoonTwoResultsChoice = {
    NOBODY: 1,
    SEKI: 2,
    FARMER: 3,
    VIXEN: 4
};
