/**
 * Hide section of the woods.
 * @param {ForestController} controller The Forest controller.
 */
function ForestWoodsWeather(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = new ForestWoodsWeatherContinue(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        global.kobold.history.shared.enslavement.beesting = true;
        var toReturn = `[p]
                            No.
                            You can't move.
                            If you move, they'll see you.
                            Your best bet would be to stay put.
                            Surely, the bees won't sting you unless you look like a threat to them, right?
                        [/p]
                        [p]
                            Wrong.
                            While your pursuers take their time to comb through every single bush in the area,
                            a trickle of bees begins to cover your body, most of them concentrating around the initial prick,
                            some of them landing on your back, and a few even flying out as far as your legs and your feet.
                            You protect your face with your arms,
                            preventing the drones from getting to your snout, or your eyes,
                            the bottom half of your muzzle pressing against the bark of the branch you're laying atop.
                        [/p]
                        [p]
                            At first, the bees seem content in roaming your form, scuttling and scampering across your scaly skin,
                            seemingly judging your motionless frame as [i]harmless[/i].
                            Then, a sudden sting at the back of your head marks the beginning of a veritable [b]onslaught[/b] of stabbing punctures.
                            A single bee driving his barbed stinger into your scales spurs on his comrades,
                            who follow their fellow worker's lead, sacrificing their lives in defence of the hive and the queen from a motionless kobold invader.
                        [/p]
                        [p]
                            All over, you are repeatedly stung, from your forearm to your shoulder, to your back.
                            Even the parts of your head that you fail to cover get viciously punctured, over and over again.
                            The pain is [b]excruciating[/b], small dosages of apitoxin adding up to make your entire body feel like it's [b]on fire[/b].
                            You grit your teeth and silently cry into your arms as you weather the horrible assault,
                            the need to [b]scream[/b] out in [b]agony[/b] rapidly mounting, yet if you do, your pursuers'll find you for sure.
                        [/p]
                        [p]
                            Nearly an entire minute of pure [b]torture[/b] passes before you finally notice the party 
                            and their dog leaving out of the tear-filled corner of your eye.
                            Still, you remain ontop of the branch for a little while longer, making sure they've left the area completely.
                            The bees may hurt,
                            but you are [b]far[/b] more scared of the [b]horrible[/b] things [i]they[/i] can inflict upon you than you are of a swarm of insects.
                        [/p]
                        [p]
                            Once you feel like you've hit the absolute [b]limit[/b] of how much pain you can take, you immediately attempt to climb down from the tree.
                            Your muscles weakened by the onslaught of apitoxin-injecting beestings, you find yourself almost [b]unable[/b] to move,
                            your limbs simply [b]failing[/b] to budge, panic mounting rapidly as you fear you might actually [b]die[/b] up here.
                            Scraping together every single last bit of [b]determination[/b] you have left, you [b]barely[/b] manage to roll to the side,
                            your form [b]sliding[/b] off of the thick branch, plummeting downwards, beckoned by the unrelenting pull of gravity.
                        [/p]
                        [p]
                            [b]Tumbling[/b] down the tree, your numb frame painfully hits several branches,
                            adding a new layer of [b]blunt trauma[/b] to your already near-unbearably bee-stung anatomy.
                            At least the multitude of drones still clinging to your scales are scared off by the heavy impacts,
                            the entire swarm, save for a few stragglers, vacating your form and returning to the hive.
                        [/p]
                        [p]
                            After smacking face-first into one final offshoot, you ultimately land on your back on the surprisingly soft ground,
                            in the middle of a blow-dampening patch of grass.
                            You are teetering on the [b]edge[/b] of consciousness, every moment bringing with it fresh [b]anguish[/b].
                            You can't move.
                            You can't think.
                            You [b]wish[/b] those bees had stung you [b]to death[/b] instead of condemning you to this [b]hell[/b].
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = ctrl.preContinue.continued() ? "disabled" : "";
        var isSelected = ctrl.preContinue.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.preContinue.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.tree.get() !== ForestWoodsTreeChoice.WEATHER) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.treeWeather.change();
        part += buildChoice();
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.tree.get() !== ForestWoodsTreeChoice.WEATHER) {
            return;
        }

        next.registerListeners();

        if (ctrl.preContinue.continued()) {
            return;
        }

        registerNextListener();
    };
}
