/**
 * Hit section of the unknown scene.
 * @param {ForestController} controller The forest controller.
 */
function ForestUnknownHit(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = `unknownHitContinueButton`;
    var toEnveloped = new ForestUnknownEnveloped(ctrl);
    var toSuccess = new ForestUnknownHitSuccess(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    function buildSwipe() {
        var toReturn = `[p]
                            Time appears to slow down as adrenaline floods your system.
                            You can see the spiky tentacle lunging towards you in slow motion.
                            It undoubtedly seeks to either hurt you or grab you.
                            But you bet it [i]isn't[/i] counting on you fighting back.
                        [/p]
                        [p]
                            You ball your hand up into a fist, and strike at the surging tentacle,
                            throwing out a widely-arced hook which [b]crashes[/b] into the side of the tendril,
                            throwing it off-course, causing it to miss your body by a large margin.
                            While at first, you are proud of your achievement, 
                            it isn't until you feel a sharp, stinging pain emanating from your hand, that you realize your folly.
                        [/p]
                        [p]
                            You just outright [b]punched[/b] nature's equivalent of [b]barbed wire[/b].
                            The brief moment during which you struck it, the tendril's thorns punctured your scales, sinking deeply into your fingers,
                            and into the tender webbing in between your knuckles.
                            You howl out in pain.
                            Clutching your bleeding hand, you take a moment to inspect the injury.
                            While some of the wounds are pretty deep, you don't think anything [i]crucial[/i] was damaged.
                            At least you can still move all of your fingers.
                        [/p]
                        [p]
                            The plant, meanwhile, has recovered from the minor blow you dealt to it.
                            Sensing an opportunity, it immediately takes advantage of your distracted state.
                            While you're still attempting to judge just [i]how badly[/i] your hand was hurt,
                            two tentacles sneakily inch along the forest floor, encircling you.
                            After crossing one another and then some, they swoop back towards the central mass of the monstrosity as a unified [i]rope[/i],
                            catching the back of your feet, yanking your legs out from underneath you.
                        [/p]
                        [p]
                            Surprised, you let out a loud yelp as you are yanked off your feet, falling backwards onto the ground,
                            your head smacking against an unfortunately placed rock while the rest of your body comes to rest onto a bed of leaves with a soft thud.
                            Dazed, you phase out for a few moments.
                            Your vision is blurry, and the entire world seems to be rocking back and forth.
                            You notice something moving, but you can't quite make out what.
                            You can feel something crawling onto your arms, wrapping around them tightly.
                            [i]Oh no.[/i]
                        [/p]`;
        toReturn += ctrl.unknown.yank.hit.change();
        return toReturn;
    }

    function buildFight() {
        var weapon = ctrl.unknown.weapon();
        var weaponPopup = global.kobold.inventory.createItemWithTooltip(weapon);
        var toReturn = `[p]
                            Time appears to slow down as adrenaline floods your system.
                            You can see the spiky tentacle lunging towards you in slow motion.
                            It undoubtedly seeks to either hurt you or grab you.
                            But you bet it [i]isn't[/i] counting on you being armed.
                        [/p]
                        [p]
                            You deftly swing your ${weaponPopup} at the surging tendril in a wide arc, 
                            and due to a combination of sheer [i]luck[/i] and adrenaline-fueled expertise,
                            you skillfully manage to [b]smack[/b] the very tip of the floral appendage with the [b]full[/b] force of your weapon.
                            You strike it [b]so[/b] hard that some of the thorns outright [b]break off[/b],
                            flying off of the tentacle like teeth out of a bar brawler's mouth, leaving sap-oozing holes in their wake.
                        [/p]
                        [p]
                            The moment your strike connects, the very passage of time seems to speed up again,
                            your temporary enhanced perception fading just as quickly as it emerged.
                            The tendril, along with the entire animated thornbush, recoils, almost visibly [i]hurt[/i].
                        [/p]
                        [p]
                            A small, cruel smirk creeps across your face.
                            While your sweater getting stuck on the bush was undeniably [i]your[/i] fault, 
                            it still feels good to [b]relieve some frustration[/b] on this monstrosity.
                            If it hadn't been [b]in your way[/b], none of this would have happened in the first place!
                            The tentacle monster recollects itself, before advancing upon you once more.
                            You brace your weapon and prepare yourself.
                            If it's a fight it wants, a fight it'll get.
                        [/p]`;
        return toReturn;
    }

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = ``;

        if (ctrl.unknown.weapon()) {
            toReturn += buildFight();
        } else {
            toReturn += buildSwipe();
        }

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.unknown.yank.hit.continued();
        var choice = `<div class="choicecontainer">
                        <button class="button inlinechoice ${disabled ? "selected" : ""}" id="${continueId}" ${disabled ? "disabled" : ""}>
                            Continue
                        </button>
                      </div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.unknown.yank.hit.continue();
            if (ctrl.unknown.weapon()) {
                ctrl.unknown.yank.hit.success();
                loadNextPart(toSuccess);
            } else {
                ctrl.unknown.yank.hit.envelop();
                loadNextPart(toEnveloped);
            }
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = "forestUnknownHit";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.unknown.yank.base.get() !== ForestUnknownYankChoice.HIT) {
            return "";
        }
        
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += buildChoice();
        part += toSuccess.build();
        part += toEnveloped.build();
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.unknown.yank.base.get() !== ForestUnknownYankChoice.HIT) {
            return;
        }

        toSuccess.registerListeners();
        toEnveloped.registerListeners();

        if (ctrl.unknown.yank.hit.continued()) {
            return;
        }

        registerContinueListener();
    };
}
