/**
 * Barn section of the farm scene.
 * @param {BreedingFarmController} controller The breeding farm controller.
 */
function NotBarnDisagree(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = "notBarnDisagreeButton";

    var next = new NotBarnDisagreeSex(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var shirt = ctrl.misc.shirt();
        var toReturn = `[p]
                            No!
                            Keeping kobolds as [b]slaves[/b] is [b]wrong[/b]!
                            You passionately express your concerns about what the vixen is doing,
                            but she seemingly [i]ignores[/i] you, instead concentrating on getting the door open.
                            You raise your voice, you even yell at her a little to grab her attention,
                            or at least to elicit [b]any[/b] kind of response, without success.
                        [/p]
                        [p]
                            To an outsider, the entire scene probably resembled a child throwing a tantrum,
                            with its mother not being in the mood to humor it.
                            Right as you're getting fed up with being ignored, as you've reached your limit,
                            as you've had about [b]enough[/b] of Cassidy not so much as acknowledging your [b]existence[/b],
                            a loud [i]click[/i] resonates from the lock she's trying to open, interrupting your outburst of anger.
                        [/p]
                        [p]
                            She pushes open the front door to her house.
                            Then, in the flash of an eye, before you even [i]know[/i] what's going on,
                            the canine reaches behind her, crudely grabbing your arm with pinpoint precision.
                            She hurls you forwards, practically [b]tossing[/b] you into her home.
                            You fail to keep your balance, landing face-first on the smooth,
                            wooden floor, sliding along it for a good foot or two before coming to a halt.
                        [/p]
                        [p]
                            You flip yourself onto your back,
                            right on time to witness Cassidy entering her own mansion as well, closing the door once more,
                            the luxurious hallway you've been cast into suddenly growing a lot more claustrophobic 
                            as the last rays of sunlight beaming in through the doorway get blocked by the wooden surface.
                            Now it's just you, her, and the multitude of lit candles and paintings lining the walls.
                        [/p]
                        [p]
                            The sultry smile on the vixen's face has been replaced by an angry glare.
                            "[color=orange][b]Uppity[/b] little [b]sluts[/b] like [b]you[/b] need to be [b]tamed[/b].[/color]" she says,
                            her tone seemingly [b]dead-serious[/b].
                            Right after those words have left her mouth, the near-naked vulpine begins to advance upon you, slowly walking towards your downed form.
                        [/p]
                        [p]
                            You panic, wildly flailing your limbs about in a desperate attempt to get away,
                            your mind torn between either crawling backwards or rising to your feet, your body doing neither.
                            Your smooth-scaled hands and feet find little to no grip on the surface of the polished floor,
                            and with nothing nearby to grab onto, you are left flailing like a fish out of water,
                            watching in horror as the woman leisurely approaches.
                        [/p]
                        [p]
                            Realizing you aren't getting away, at least not like this, you cease your frantic squirming.
                            You try to think of another way out, of a plan of action, of a way to escape, yet it's already too late.
                            A heavy leather boot plops down onto your chest, [i]almost[/i] [b]crushing[/b] the air out of you,
                            the vixen grateful enough [b]not[/b] to stomp down with all of her might, or even to lean her entire weight onto you,
                            applying [i]just[/i] enough pressure to keep you pinned down, and nothing more.
                        [/p]
                        [p]
                            You consider fighting against her foot-hold, yet the sharp spur at the back of her boot, which is mere inches away from
                            ${shirt ? `tearing into your ${global.kobold.inventory.createItemWithTooltip(shirt)}` : `carving into your flesh`},
                            dissuades you from struggling.
                            Instead, you lie limply, staring up at Cassidy with fear in your eyes.
                            She leans over you like a hunter would over felled prey, her toothy, predatory grin only serving to make you even [b]more[/b] afraid.
                        [/p]
                        [p]
                            Staring down at you, the woman speaks:
                            "[color=orange]You promised me a good time, and that's [b]exactly[/b] what I intend on having.
                            I don't care how much you [b]hate[/b] my guts.
                            You're going to do [b]exactly[/b] as I say,
                            and then maybe I'll consider letting you [b]leave[/b] instead of hogtying you,
                            throwing you into the barn, and showing you exactly how [b]depraved[/b] your kind can get.[/color]"
                        [/p]
                        [p]
                            You swallow deeply, the true severity of the situation only now dawning upon you.
                            You continue to look for an exit plan, for a way to wriggle out from underneath her, yet you fail to come up with anything.
                            You are trapped.
                            Stuck between a boot and a hard place.
                            There [b]is[/b] no way out.
                        [/p]
                        [p]
                            "[color=orange]So, [b]lizard[/b].
                            What'll it be?
                            Are you going to be a good ${ctrl.misc.baseGender() === GenderEnum.MALE ? "boy" : "girl"}?
                            Or am I going to have to [b]punish[/b] you?[/color]"
                        [/p]
                        [p]
                            Panic turning to unwillingness, you look away to the side for a few moments,
                            simply staring at the bottom of the wall while your brain struggles to decide.
                            A single tear spills from your eyes, rolling down over your scales, dropping to the ground.
                            Your teary gaze wanders back towards Cassidy, who is patiently awaiting your reply.
                            She knows she's fully in control of the situation.
                            And you, too, are painfully well aware of that fact.
                            Your fighting spirit broken, you give her but a single, defeated nod.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildContinueChoice() {
        var disabled = ctrl.postnotbarn.has() ? "disabled" : "";
        var isSelected = ctrl.postnotbarn.has();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildContinueChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.postnotbarn.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = "notBarnDisagree";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.notbarn.get() !== NotBarnChoice.DISAGREE) {
            return "";
        }
        global.ui.applyTheme("hamlet_mansion");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.notbarnDisagreeChange();
        part += buildChoice();
        part += next.build();
        part += `</div>`;
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.notbarn.get() !== NotBarnChoice.DISAGREE) {
            return;
        }

        next.registerListeners();

        if (ctrl.postnotbarn.has()) {
            return;
        }

        registerContinueListener();
    };
}
