/**
 * Confront section of the Farmer House scene.
 * @param {FarmerHouseController} controller The Farmer House controller.
 */
function FarmerHousePartialLoot(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = arguments.callee.name + "ButtonOne";
    var next = new FarmerHouseLootContinue(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            With the woman in check, you decide it's time to do what you came here to do: loot whatever you can.
                            Reminding her once more to [b]stay down[/b], you step towards the kitchen counter.
                            Keeping an eye on the housewife, you begin to feel around for things to steal.
                            Sadly, the countertop contains pretty much [b]nothing[/b] of value.
                            The only thing your hand comes across is the ${global.kobold.inventory.createItemWithTooltip(new Bread())},
                            the ${global.kobold.inventory.createItemWithTooltip(new Butter())}
                            and the ${global.kobold.inventory.createItemWithTooltip(new Ham())}
                            the woman had brought with her.
                        [/p]
                        [p]
                            You can tell that the housewife, while still pacified for the moment being, is just [b]waiting[/b] for you to slip up,
                            so she can run, scream for help or perhaps even [b]attack[/b] you.
                            You don't dare go into the next room to see if the pickings there are better.
                            Not wanting to leave here empty-handed either, you decide to take the peasant family's food.
                        [/p]
                        [p]
                            You pocket the bread, the butter and the ham.
                            Frustrated that you weren't able to steal more, but afraid of what the woman might do, should you leave her out of your sight,
                            you slowly back away into the hallway from which you came.
                            After casting one last menacing glare her way, you face away from the housewife, and [b]bolt[/b] out of the backdoor as fast as you can.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildNextChoice() {
        var disabled = ctrl.partialLoot.continued() ? "disabled" : "";
        var isSelected = ctrl.partialLoot.continued();
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${continueId}" ${disabled}>Continue</button>`;
        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer">`;
        choice += buildNextChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerNextListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.partialLoot.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.scared.get() !== FarmerHouseScaredChoice.LOOT) {
            return "";
        }
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += ctrl.partialLoot.change();
        part += buildChoice();
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.scared.get() !== FarmerHouseScaredChoice.LOOT) {
            return;
        }

        next.registerListeners();

        if (ctrl.partialLoot.continued()) {
            return;
        }

        registerNextListener();
    };
}
