/**
 * Arrival section of the Workshop scene.
 * @param {SekiWorkshopController} controller The Seki controller.
 */
function SekiWorkshopArrival(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var continueId = "arrivalContinueButton";

    var next = new SekiWorkshopEntrance(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            You follow the eastern road towards Varanar,
                            making sure to keep your eyes peeled for any would-be assailants.
                            Luckily, your stroll is quite uneventful.
                            Bandits usually operate closer to nighttime.
                            Most of them must still be asleep.
                            It doesn't take you long to reach the small,
                            branching dirt path that'll lead you directly towards the hamlet.
                        [/p]
                        [p]
                            Approaching the small village, you cross paths with a couple of farmers.
                            You figure they're either making their way to their respective fields, or heading to Varanar for whatever reason.
                            They don't seem to be carrying any goods for sale, not that it's any of your business.
                            The humans are too busy speaking to one another to pay you much mind.
                            Shrugging, you continue along your journey.
                        [/p]
                        [p]
                            Around noon, you arrive at the hamlet.
                            The sun is high up in the sky, although the temperature remains somewhat frigid.
                            The settlement you have entered considers of nothing but a few meagre houses, a [i]breeding farm[/i], 
                            and a single shop, ran by an old friend of yours.
                            That last place is where you decide to head towards first.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the continue button.
     */
    function buildChoice() {
        var disabled = ctrl.arrival.continued();
        var choice = `<div class="choicecontainer">
                        <button class="button inlinechoice ${disabled ? "selected" : ""}" id="${continueId}" ${disabled ? "disabled" : ""}>
                            Continue
                        </button>
                      </div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + continueId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerContinueListener() {
        $("#" + continueId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.arrival.continue();
            loadNextPart(next);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = "sekiArrival";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        global.ui.applyTheme("hamlet_day");
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += buildChoice();
        part += `</div>`;
        part += next.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        next.registerListeners();

        if (ctrl.arrival.continued()) {
            return;
        }

        registerContinueListener();
    };
}
