/**
 * Fight section of the Underground Cave.
 * @param {UndergroundCaveController} controller The Underground Cave controller.
 */
function UndergroundCaveWrestlingWin(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var reqChoice = false;
    var reqController = false;
    var subController = ctrl.wrestleWin;
    var changeController = ctrl.wrestleWin;

    var firstId = arguments.callee.name + "ButtonOne";
    var secondId = arguments.callee.name + "ButtonTwo";
    var thirdId = arguments.callee.name + "ButtonThree";

    var firstNext = new UndergroundCaveSnap(ctrl);
    var secondNext = new UndergroundCaveLetGo(ctrl);
    var thirdNext = new UndergroundCaveDominance(ctrl);

    var firstChoice = UndergroundCaveWrestlingWinChoice.SNAP;
    var secondChoice = UndergroundCaveWrestlingWinChoice.GO;
    var thirdChoice = UndergroundCaveWrestlingWinChoice.DOMINANCE;

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var toReturn = `[p]
                            The basilisk thinks it's tough.
                            And, admittedly, being made out of mostly stone, it most likely [b]is[/b].
                            Heck, it probably weighs several times more than you do.
                            But you have something it doesn't.
                            You have evolved [b]beyond[/b] being a [b]petty lizard[/b].
                            It might be able to swing its brute limbs around with unmatched ferocity,
                            but what you lack in strength, you make up for in wits.
                        [/p]
                        [p]
                            Your brain kicks into overdrive, a million thoughts flashing by in the blink of an eye.
                            Its hide is made out of rock.
                            You can't [b]hit it[/b], you'd just end up breaking your hand.
                            Your claws won't penetrate stone.
                            You don't have a weapon to use, and there aren't any loose rocks around.
                            You could try to lure it into the water, but that's a longshot at best.
                            Besides, it [i]might[/i] be able to swim.
                            That leaves only one option.
                            And it's [b]risky[/b].
                        [/p]
                        [p]
                            After a tense moment of inactivity, the monster suddenly [b]charges[/b] at you, bursting into a sprint.
                            It tries to bite you, but you deftly step to the side, avoiding its barreling frame.
                            The creature's powerful jaws snap shut, catching nothing but air.
                            You spot the perfect opportunity.
                            Here it goes.
                            All or nothing.
                        [/p]
                        [p]
                            You [b]leap[/b] ontop of the crocodillian's back, instantly grabbing its throat in a choke-hold.
                            The bottom of the beast feels a lot [b]squishier[/b] than the top.
                            Perhaps its soft belly is its weak spot?
                        [/p]
                        [p]
                            You are in the [b]perfect[/b] position.
                            While ontop of the basilisk, it can't hit you with its claws, it can't smack you with its tail, nor can it bite you.
                            Instead of outright accosting you, the monster begins to thrash about, wildly shaking its body in an attempt to throw you off.
                            You hold on for dear life, wrapping a second arm around the creature's neck, squeezing its windpipe shut while clinging to its back.
                        [/p]
                        [p]
                            Realizing you aren't going to let go like this, the basilisk switches tactics.
                            For a few moments, it seemingly calms down, its muscles tangibly [b]tensing[/b].
                            Right as thought it was on the verge of [b]surrendering[/b],
                            the beast [b]rolls[/b] onto its side, and then onto its back,
                            [b]crushing[/b] you beneath its heavy, stone frame.
                        [/p]
                        [p]
                            You [b]scream[/b] out in pain, but you do [b]not[/b] let go.
                            The torment lasts for all but a second, the basilisk continuing to roll around,
                            returning to its feet before tumbling to its back yet again,
                            [b]squishing[/b] you between its rocky hide and the stone cavern floor a second time.
                            No wonder they call this the [b]death roll[/b].
                        [/p]
                        [p]
                            The massive monster rolls around three, no, [b]four[/b] times before grinding to a halt.
                            Despite everything, you've managed to stay ontop of it, but only [b]barely[/b].
                            [b]Every[/b] part of your body [b]hurts[/b].
                            You can't handle any more.
                            You feel like if the monster rolls onto its back one more time, you'll [b]break[/b].
                        [/p]
                        [p]
                            You are breathing heavily.
                            It, too, [b]tries[/b] to catch some breath, but your forearm is still deftly blocking the creature's windpipe.
                            While it's done rolling, for now, it still does [b]not[/b] yield.
                            You [b]need[/b] to end this [b]now[/b].
                            If it gathers enough energy to start rolling again, you won't survive.
                        [/p]
                        [p]
                            Keeping one arm around the basilisk's neck,
                            you begin to wantonly [b]punch[/b] its underside with the other,
                            repeatedly bashing the feral from the bottom of its maw to its soft and vulnerable belly.
                            After about half a minute more of choking it while hitting it with your fist,
                            the beast finally slumps down to the ground, letting out a submissive whimper.
                        [/p]
                        [p]
                            [c]
                                [b]You have beaten the basilisk![/b]
                            [/c]
                        [/p]`;

        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the choice button.
     */
    function buildFirstChoice() {
        var disabled = subController.has() ? "disabled" : "";
        var isSelected = subController.get() === firstChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${firstId}" ${disabled}>Kill it</button>`;

        var tooltip = global.parser.parse(`This has gone on for long enough. End the basilisk.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 190px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildSecondChoice() {
        var disabled = subController.has() ? "disabled" : "";
        var isSelected = subController.get() === secondChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${secondId}" ${disabled}>Let it go</button>`;

        var tooltip = global.parser.parse(`The fight's over. You won. There's no need to murder the creature.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 190px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice button.
     */
    function buildThirdChoice() {
        var disabled = subController.has() ? "disabled" : "";
        var isSelected = subController.get() === thirdChoice;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${thirdId}" ${disabled} style="width:200px;">Assert dominance</button>`;

        var male = ctrl.misc.dick() ? "male" : "female";
        var tooltip = global.parser.parse(`The fight is over. You won. Time to assert yourself as the alpha ${male}.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 190px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice" style="width:200px;">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You:</div>`;
        choice += buildFirstChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildSecondChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildThirdChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + firstId), $("#" + secondId), $("#" + thirdId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerFirstListener() {
        $("#" + firstId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(firstChoice);
            loadNextPart(firstNext);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerSecondListener() {
        $("#" + secondId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(secondChoice);
            loadNextPart(secondNext);
        });
    }

    /**
     * Adds a listener to the button.
     */
    function registerThirdListener() {
        $("#" + thirdId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            subController.set(thirdChoice);
            loadNextPart(thirdNext);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = arguments.callee.name;

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return "";
        }

        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += (changeController && changeController.change()) || "";
        part += buildChoice();
        part += `</div>`;
        part += firstNext.build();
        part += secondNext.build();
        part += thirdNext.build();
        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (reqController && ((reqChoice && reqController.get() !== reqChoice) || (!reqChoice && !reqController.continued()))) {
            return;
        }

        firstNext.registerListeners();
        secondNext.registerListeners();
        thirdNext.registerListeners();

        if (subController.has()) {
            return;
        }

        registerFirstListener();
        registerSecondListener();
        registerThirdListener();
    };
}

var UndergroundCaveWrestlingWinChoice = {
    SNAP: 1,
    GO: 2,
    DOMINANCE: 3
};
