/**
 * Controller for the saved part of the home scene.
 * @param {string} sceneId The id of the scene.
 */
function HomeSavedController(sceneId) {
    // PRIVATE MEMBERS \\       var private = "test";
    var id = sceneId;
    var approachId = "savedApproach";
    var debtId = "savedDebt";
    var weaponId = "savedWeapon";
    var ambushId = "savedAmbush";
    var hideId = "savedHide";

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    // PUBLIC MEMBERS \\        this.public = "test";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Makes the approach choice.
     * @param {HomeSavedChoice} choice The choice.
     */
    this.makeApproachChoice = function (choice) {
        if (choice === HomeSavedChoice.AMBUSH) {
            global.kobold.history.shared.debtCollector.status = DebtCollectorEnum.AMBUSHED;
        }
        if (choice === HomeSavedChoice.HIDE) {
            global.kobold.history.shared.debtCollector.status = DebtCollectorEnum.UNPAID;
        }
        global.kobold.history.setKey(id, approachId, choice);
    };

    /**
     * Fetches the approach choice from history and returns it.
     * May return undefined if the approach choice has not yet been set.
     */
    this.getApproachChoice = function () {
        return global.kobold.history.getKey(id, approachId);
    };

    /**
     * Returns true if the approach choice has been made.
     */
    this.hasMadeApproachChoice = function () {
        return global.kobold.history.has(id, approachId);
    };


    function DebtController(key) {
        var segment = key + "first";
        var acceptChangeId = key + "acceptChange";
        var refuseChangeId = key + "refuseChange";
        var acceptWakeChangeId = key + "acceptWakeChange";
        var refuseWakeChangeId = key + "refuseWakeChange";
        this.get = function () {
            return global.kobold.history.getKey(id, key);
        };
        this.set = function (choice) {
            if (choice === HomeApproachChoice.ACCEPT) {
                global.kobold.history.shared.debtCollector.status = DebtCollectorEnum.PAID;
            }
            if (choice === HomeApproachChoice.REFUSE) {
                global.kobold.history.shared.debtCollector.status = DebtCollectorEnum.REFUSED;
            }
            global.kobold.history.setKey(id, key, choice);
        };
        this.has = function () {
            return global.kobold.history.has(id, key);
        };
        this.firstSegmentContinue = function () {
            global.kobold.history.setKey(id, segment, true);
        };
        this.firstSegmentContinued = function () {
            return global.kobold.history.has(id, segment);
        };
        this.acceptChange = function (coins) {
            global.kobold.history.doIfNotSet(id, acceptChangeId, true, function () {
                global.kobold.history.shared.debtCollector.paid += coins;
                global.kobold.inventory.coins = 0;
                global.kobold.status.mental.increase(20, "Immense relief");
                global.kobold.stats.increaseStat(StatsEnum.INTELLIGENCE, 1);
            });
            return global.ui.change.buildNotification([
                new Change("Debt paid", "Coins", -coins),
                new Change("Immense relief", "Sanity", +20),
                new Change("Mature conduct", "Intelligence", +1)
            ]);
        };
        this.refuseChange = function (coins) {
            global.kobold.history.doIfNotSet(id, refuseChangeId, true, function () {
                global.kobold.history.shared.debtCollector.paid += coins;
                global.kobold.inventory.coins = 0;
                global.kobold.status.physical.decrease(20, "Punched");
                global.kobold.status.mental.decrease(5, "Bravery");
            });
            return global.ui.change.buildNotification([
                new Change("Money taken", "Coins", -coins),
                new Change("Punched", "Health", -20),
                new Change("Money taken", "Sanity", -10),
                new Change("Bravery", "Sanity", +5)
            ]);
        };
        this.acceptWakeChange = function () {
            global.kobold.history.doIfNotSet(id, acceptWakeChangeId, true, function () {
                global.kobold.status.physical.increase(40, "Well-earned nap");
                global.kobold.status.stamina.increase(40, "Well-earned nap");
            });
            return global.ui.change.buildNotification([
                new Change("Well-earned nap", "Health", +40),
                new Change("Well-earned nap", "Stamina", +40)
            ]);
        };
        this.refuseWakeChange = function () {
            global.kobold.history.doIfNotSet(id, refuseWakeChangeId, true, function () {
                global.kobold.status.physical.increase(10, "Jaw-pain subsided");
                global.kobold.status.mental.decrease(10, "Troubled sleep");
                global.kobold.status.stamina.increase(20, "Mediocre rest");
            });
            return global.ui.change.buildNotification([
                new Change("Pain subsided", "Health", +10),
                new Change("Troubled sleep", "Sanity", -10),
                new Change("Mediocre rest", "Stamina", +20)
            ]);
        };
    }

    this.debt = new DebtController(debtId);

    function AmbushController(key) {
        var knockChangeId = key + "KnockChange";
        var knockId = key + "Knocked";
        var sleepChangeId = key + "SleepChange";
        var sleepId = key + "Sleep";
        var wakeChangeId = key + "WakeChange";
        this.knockChange = function () {
            global.kobold.history.doIfNotSet(id, knockChangeId, true, function () {
                global.kobold.status.physical.decrease(40, "Knocked out");
            });
            return global.ui.change.buildNotification([
                new Change("Knocked out", "Health", -40)
            ]);
        };
        this.knockContinue = function () {
            global.kobold.history.setKey(id, knockId, true);
        };
        this.knockContinued = function () {
            return global.kobold.history.has(id, knockId);
        };
        this.sleepChange = function (coins) {
            global.kobold.history.doIfNotSet(id, sleepChangeId, true, function () {
                global.kobold.history.shared.debtCollector.paid += coins;
                global.kobold.inventory.coins = 0;
                global.kobold.status.mental.decrease(20, "Brain damage");
            });
            return global.ui.change.buildNotification([
                new Change("Debt collected", "Coins", -coins),
                new Change("Brain damage", "Sanity", -20)
            ]);
        };
        this.sleepContinue = function () {
            global.kobold.history.setKey(id, sleepId, true);
        };
        this.sleepContinued = function () {
            return global.kobold.history.has(id, sleepId);
        };
        this.wakeChange = function () {
            global.kobold.history.doIfNotSet(id, wakeChangeId, true, function () {
                global.kobold.status.physical.increase(10, "Wounds healing");
                global.kobold.status.stamina.increase(20, "Pained slumber");
                global.kobold.stats.decreaseStat(StatsEnum.AGILITY, 1);
                global.kobold.stats.decreaseStat(StatsEnum.INTELLIGENCE, 1);
            });
            return global.ui.change.buildNotification([
                new Change("Wounds healing", "Health", +10),
                new Change("Pained slumber", "Stamina", +20),
                new Change("Permanent damage", "Agility", -1),
                new Change("Permanent damage", "Intelligence", -1)
            ]);
        };
    }

    this.ambush = new AmbushController(ambushId);

    function HideController(key) {
        var ballId = key + "Ball";
        var ballChangeId = key + "BallChange";
        var sleepChangeId = key + "SleepChange";
        var sleepId = key + "Sleep";
        var wakeChangeId = key + "WakeChange";
        this.ballChange = function () {
            global.kobold.history.doIfNotSet(id, ballChangeId, true, function () {
                global.kobold.status.physical.decrease(70, "Brutal beatdown");
            });
            return global.ui.change.buildNotification([
                new Change("Brutal beatdown", "Health", -70)
            ]);
        };
        this.ballContinue = function () {
            global.kobold.history.setKey(id, ballId, true);
        };
        this.ballContinued = function () {
            return global.kobold.history.has(id, ballId);
        };
        this.sleepChange = function () {
            global.kobold.history.doIfNotSet(id, sleepChangeId, true, function () {
                global.kobold.status.mental.decrease(10, "Brutal beatdown");
            });
            return global.ui.change.buildNotification([
                new Change("Kept coins", "Sanity", +10),
                new Change("But at what cost?", "Sanity", -20)
            ]);
        };
        this.sleepContinue = function () {
            global.kobold.history.setKey(id, sleepId, true);
        };
        this.sleepContinued = function () {
            return global.kobold.history.has(id, sleepId);
        };
        this.wakeChange = function () {
            global.kobold.history.doIfNotSet(id, wakeChangeId, true, function () {
                global.kobold.status.physical.increase(20, "Wounds healing nicely");
                global.kobold.status.mental.increase(10, "Cautious optimism");
                global.kobold.status.stamina.increase(20, "Pained slumber");
                global.kobold.stats.increaseStat(StatsEnum.ENDURANCE, 1);
            });
            return global.ui.change.buildNotification([
                new Change("Wounds healing", "Health", +20),
                new Change("Cautious optimism", "Sanity", +10),
                new Change("Pained slumber", "Stamina", +20),
                new Change("What doesn't kill you", "Endurance", +1)
            ]);
        };
    }

    this.hide = new HideController(hideId);

    /**
     * Stored and returns the kobold's weapon.
     */
    this.weapon = function () {
        global.kobold.history.setIfNotSet(id, weaponId, global.kobold.inventory.equipment.weapon);
        return global.kobold.inventory.equipment.buildGearFromHistory(global.kobold.history.getKey(id, weaponId));
    };
}

