/**
 * Ground sleeping section of the home scene.
 * @param {HomeController} controller The home controller.
 */
function HomeGround(controller) {
    // PRIVATE MEMBERS \\       var private = "test";
    var that = this;
    var ctrl = controller;

    var antId = "unsavedAntButton";
    var shakeId = "unsavedShakeButton";

    var toAnt = new HomeAnt(ctrl);
    var toShake = new HomeShake(ctrl);

    // PRIVATE FUNCTIONS \\     function privateFunction(){}

    /**
     * Builds and returns the content of this section.
     * @returns {string} The content.
     */
    function buildContent() {
        var male = ctrl.gender() === GenderEnum.MALE;
        var toReturn = `[p]
                            You owe the clan [b]a lot[/b] of money.
                            You wouldn't be surprised if the Three-Tailed Rat were to send some kind of night-time 
                            assassination squad to your home to try and [i]teach you[/i] a [b]lethal late-night lesson[/b].
                        [/p]
                        [p]
                            You can't risk sleeping in your home. That's the first place they'd come looking for you.
                            While using the hammock is a nice idea, you'd be [b]far[/b] too exposed.
                            You decide to choose [b]safety[/b] over [i]comfort[/i].
                            The ground is where you'll sleep tonight.
                        [/p]
                        [p]
                            You wander a little further into the woods, away from your shed.
                            After a solid minute of walking, you stumble upon a bush large enough to conceal your entire body.
                            You sigh loudly, getting flashbacks from your younger days as you reluctantly lower yourself into the foliage,
                            your small frame coming to rest on the cold ground at the very root of the shrub.
                        [/p]
                        [p]
                            While by no means warm, at least the plant's leaves somewhat protect you from the freezing cold washing over the land.
                            This isn't your first time sleeping on the forest floor, but your mind is too weary to reminisce right now.
                            Curling up into a ball, you slowly drift to sleep.
                        [/p]
                        [p]
                            You wake up far earlier than you'd have liked, to the feeling of something small [i]crawling[/i] over your scales.
                            At first you think you're dreaming, but the sensation persist.
                            It moves along your exposed belly, creeping every which way.
                        [/p]
                        [p]
                            As you open your heavy eyelids and stare down at your body, you are greeted with the cause of the odd sensation.
                            A bunch of ants are crawling about your lower body, many more swarming out of a nearby anthill that you failed to spot earlier.
                            Worse yet, these aren't just regular ants.
                            Even in the near pitch-blackness of night, you can make out their distinctly [color=pink]pink[/color] hue.
                            These are the dreaded [b]Venusian ants[/b], 
                            whose bite is said to inject a potent aphrodisiac strong enough for ten of them to make a male horse erect.
                        [/p]
                        [p]
                            Your mind instantly snaps awake as one of the small bugs digs its pincers into your scales.
                            It doesn't [i]hurt[/i] more than a small pinch, yet your groin instantly responds to the venom,
                            ${male ? "your member stirring to life inside of your cockslit" : "your slit growing just a little more moist"}.
                        [/p]`;
        return global.parser.parse(toReturn);
    }

    /**
     * Builds and returns the ant choice button.
     */
    function buildAntChoice() {
        var disabled = ctrl.unsaved.hasMadeGroundChoice() ? "disabled" : "";
        var isSelected = ctrl.unsaved.getGroundChoice() === HomeGroundChoice.ANT;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${antId}" ${disabled}>Want more</button>`;

        var tooltip = global.parser.parse(`The aphrodisiac feels good. [b]So[/b] good. You want [b]more[/b] of their venom. You [b]need[/b] more of it.`);
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 210px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the shake choice button.
     */
    function buildShakeChoice() {
        var disabled = ctrl.unsaved.hasMadeGroundChoice() ? "disabled" : "";
        var isSelected = ctrl.unsaved.getGroundChoice() === HomeGroundChoice.SHAKE;
        var selected = isSelected ? "selected" : "";
        var button = `<button class="button inlinechoice ${selected}" id="${shakeId}" ${disabled} style="font-size: 18px;">Shake them off</button>`;

        var tooltip = `Get the little things off of you before they can do more harm!`;
        var popupContainer = global.ui.makeToolTip(button, tooltip, false, false, "width: 180px;");
        // Re-wrap with proper positioning.
        button = `<div class="inlinechoice">`;
        button += popupContainer;
        button += `</div>`;

        return button;
    }

    /**
     * Builds and returns the choice of this section.
     * @returns {string} The choice of this section.
     */
    function buildChoice() {
        var choice = `<div class="choicecontainer"><div class="inlinechoiceheader">You: </div>`;
        choice += buildAntChoice();
        choice += `<hr class="inlinechoiceseparator">`;
        choice += buildShakeChoice();
        choice += `</div>`;
        return choice;
    }

    /**
     * Disables all three buttons.
     */
    function disableButtons() {
        var buttons = [$("#" + antId), $("#" + shakeId)];
        buttons.forEach(function (button) {
            button.unbind();
            button.attr("disabled", true);
        });
    }

    /**
     * Loads and displays the next part.
     * @param {object} part Which part to load.
     */
    function loadNextPart(part) {
        var built = part.build(true);
        global.ui.appendBeforeFinisher(built, global.ui.genericHeaderId);
        $("#" + part.id).fadeIn(400);
        part.registerListeners();
    }

    /**
     * Adds a listener to the button.
     */
    function registerAntListener() {
        $("#" + antId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.unsaved.makeGroundChoice(HomeGroundChoice.ANT);
            loadNextPart(toAnt);
        });

    }

    /**
     * Adds a listener to the button.
     */
    function registerShakeListener() {
        $("#" + shakeId).click(function () {
            $(this).addClass("selected");
            disableButtons();
            ctrl.unsaved.makeGroundChoice(HomeGroundChoice.SHAKE);
            loadNextPart(toShake);
        });
    }

    // PUBLIC MEMBERS \\        this.public = "test";
    this.id = "homeGround";

    // PUBLIC FUNCTIONS \\      this.publicFunction = function(){};

    /**
     * Builds and returns this part.
     * @param {boolean} invisible If passed and true, will give the entire section a display:none style.
     */
    this.build = function (invisible) {
        if (ctrl.unsaved.getSleepChoice() !== HomeUnsavedChoice.GROUND) {
            return "";
        }
        var part = `<div id="${that.id}" ${invisible ? 'style="display:none;"' : ""}>`;
        part += buildContent();
        part += buildChoice();
        part += toAnt.build();
        part += toShake.build();
        part += `</div>`;

        // BUILD CHILDREN HERE

        return part;
    };

    /**
     * Registers all the required button listeners for this part.
     */
    this.registerListeners = function () {
        if (ctrl.unsaved.getSleepChoice() !== HomeUnsavedChoice.GROUND) {
            return;
        }

        toAnt.registerListeners();
        toShake.registerListeners();

        if (ctrl.unsaved.hasMadeGroundChoice()) {
            return;
        }

        registerAntListener();
        registerShakeListener();
    };
}

var HomeGroundChoice = {
    ANT: 1,
    SHAKE: 2
};
